import { NbMenuItem } from "@nebular/theme";
import { SubatoPrincipal } from "../models/principal/subato-principal";

export class TopMenu {
  public static getMenu(currentUrl: string, courseId: number | null, principal: SubatoPrincipal | null): NbMenuItem[] {
    return [
      this.coursesItem(currentUrl, courseId),
      this.tasksItem(currentUrl, courseId, principal),
      this.adminItem(currentUrl, principal)
    ];
  }

  public static coursesItem(currentUrl: string, courseId: number | null): NbMenuItem {
    return {
      title: 'Kurse',
      icon: 'compass-outline',
      link: '/course',
      selected: currentUrl.startsWith('/course') || courseId != null
    };
  }

  public static tasksItem(currentUrl: string, courseId: number | null, principal: SubatoPrincipal | null): NbMenuItem {
    return {
      title: 'Aufgaben',
      icon: 'grid-outline',
      link: `/task_pools`,
      hidden: !(principal != null && (principal.hasStaffRole() || principal.hasSuperAdminRole())),
      selected: currentUrl.startsWith('/task_pools') ||
        currentUrl.startsWith('/tasks') ||
        (courseId == null && currentUrl.startsWith('/solutions')),
      data: { topNav: true }
    };
  }

  public static adminItem(currentUrl: string, principal: SubatoPrincipal | null): NbMenuItem {
    let children = [
      {
        title: 'Semester',
        icon: 'calendar-outline',
        link: `/term`,
        selected: currentUrl.startsWith('/term')
      },
      {
        title: 'Sprachen',
        icon: 'code-outline',
        link: `/language`,
        selected: currentUrl.startsWith('/language')
      }
    ];

    return {
      title: 'Verwaltung',
      icon: 'settings-2-outline',
      hidden: !(principal != null && (principal.hasStaffRole() || principal.hasSuperAdminRole())),
      selected: children.some(c => c.selected),
      children: children
    };
  }
}