import { Component, Input, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AuthService } from 'app/core/services/auth.service';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest, forkJoin } from 'rxjs';
import { CourseUnitEvent } from '../../models/course-unit-event';
import { takeUntil, flatMap } from 'rxjs/operators';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { CourseUnitService } from '../../services/course-unit.service';
import { CourseUnit } from '../../models/course-unit';
import { BackendError } from 'app/shared/models/backend-error';
import { CourseUnitEventService } from '../../services/course-unit-event.service';

@Component({
  selector: 'ngx-course-unit-events',
  templateUrl: './course-unit-events.component.html',
  styleUrls: ['./course-unit-events.component.scss']
})
export class CourseUnitEventsComponent extends BaseComponent implements OnInit {
  loading: boolean;
  error: BackendError;
  @Input() unit: CourseUnit;
  events: CourseUnitEvent[] = [];
  processingEvent: CourseUnitEvent | null = null;

  constructor(
    private courseUnitService: CourseUnitService,
    private courseUnitEventService: CourseUnitEventService,
    private notificationService: NotificationService,
    private route: ActivatedRoute,
    private authService: AuthService,
  ) {
    super();
  }

  ngOnInit(): void {
    this.loading = true;

    this.courseUnitService.getEvents(this.unit.id)
      .pipe(
        takeUntil(this.unsubscribe),
      )
      .subscribe(events => {
        this.events = events;
        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
          this.notificationService.showError(
            error.format(
              "Fehler beim Laden der Termine",
            )
          );
        } else {
          throw error;
        }
      });
  }

  isCancelled(event: CourseUnitEvent): boolean {
    return event.holiday;
  }

  updateCancellation(event: CourseUnitEvent, cancel: boolean) {
    this.processingEvent = event;
    this.courseUnitEventService.updateCancellation(event.id, {
      canceled: cancel,
    }).pipe(takeUntil(this.unsubscribe))
      .subscribe(updatedEvent => {
        let idx = this.events.map(e => e.id).indexOf(updatedEvent.id);
        if(idx != -1) {
          this.events[idx].holiday = updatedEvent.holiday;
        }

        this.processingEvent = null;
        this.notificationService.showSuccess("Termin erfolgreich geändert");
      }, (error: any) => {
        this.processingEvent = null;

        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(
            error.format(
              "Fehler bei Änderung des Termins",
            )
          );
        } else {
          throw error;
        }
      });
  }
}
