import { Component, Input, OnInit } from '@angular/core';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { User } from 'app/core/models/user';
import { AuthService } from 'app/core/services/auth.service';
import { Course } from 'app/features/courses/models/course';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { BackendError } from 'app/shared/models/backend-error';
import { takeUntil, map, flatMap } from 'rxjs/operators';
import { Exercise } from '../../models/exercise';
import { ExerciseService } from '../../services/exercise.service';

@Component({
  selector: 'ngx-exercise-list',
  templateUrl: './exercise-list.component.html',
  styleUrls: ['./exercise-list.component.scss']
})
export class ExerciseListComponent extends BaseComponent implements OnInit {
  @Input() course: Course;

  exercises: Exercise[] = []
  loading: boolean;
  error: BackendError;
  principal: SubatoPrincipal;

  constructor(
    private authService: AuthService,
    private exerciseService: ExerciseService) {
    super();
  }

  ngOnInit(): void {
    this.loading = true;
    this.authService.principal$
    .pipe(
      takeUntil(this.unsubscribe),
      flatMap(principal => {
        this.principal = principal;
        return this.exerciseService.getForCourse(this.course.id);
      })
    ).subscribe(exercises => {
        this.exercises = exercises;
        this.loading = false;
        console.log(this.exercises);
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }
}
