import { Component, Input, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { TaskInstance } from '../../models/task-instance';
import { ExerciseProgress } from '../../models/exercise-progress';
import { Subject } from 'rxjs';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { takeUntil, debounceTime } from 'rxjs/operators';

@Component({
  selector: 'ngx-exercise-progress',
  templateUrl: './exercise-progress.component.html',
  styleUrls: ['./exercise-progress.component.scss']
})
export class ExerciseProgressComponent extends BaseComponent implements OnInit {
  @Input() _exerciseProgresses: ExerciseProgress[] = [];
  filteredExerciseProgresses: ExerciseProgress[] = [];
  @Input() showFilter: boolean;
  hideInvisible = true;

  query: string;
  private queryChanged: Subject<string> = new Subject<string>();
  
  @Input() set exerciseProgresses(val: ExerciseProgress[]) {
    this._exerciseProgresses = val;
    this.applyFilter();
  }

  get exerciseProgresses(): ExerciseProgress[] {
    return this._exerciseProgresses;
  }

  constructor(private router: Router) {
    super();
  }

  ngOnInit(): void {
    this.queryChanged
      .pipe(
        takeUntil(this.unsubscribe),
        debounceTime(500),
      )
      .subscribe(query => {
        this.query = query;
        this.applyFilter();
      });
  }

  navigateToSubmission(exerciseProgress: ExerciseProgress) {
    if(!exerciseProgress.submission) return;
    this.router.navigate(['/', 'submissions', exerciseProgress.submission.id]);
  }

  hasSolution(exerciseResult: ExerciseProgress, taskInstance: TaskInstance) {
    return this.getSolution(exerciseResult, taskInstance) != null;
  }

  getSolution(exerciseResult: ExerciseProgress, taskInstance: TaskInstance) {
    return exerciseResult.solutions.find(s => s.task.id == taskInstance.task.id);
  }

  hasPointRating(exerciseResult: ExerciseProgress, taskInstance: TaskInstance) {
    return this.getPointRating(exerciseResult, taskInstance) != null;
  }

  getPointRating(exerciseResult: ExerciseProgress, taskInstance: TaskInstance) {
    return exerciseResult.submission?.pointRatings.find(r => r.taskInstance.task.id == taskInstance.task.id);
  }

  onHideInvisibleChanged(val: boolean) {
    this.hideInvisible = val;
    this.applyFilter();
  }

  onQueryChange(query) {
    this.queryChanged.next(query);
  }

  applyFilter() {
    this.filteredExerciseProgresses = this.hideInvisible 
      ? this.exerciseProgresses.filter(r => !this.hideInvisible ||  r.exercise.visible)
      : this.exerciseProgresses;
    
    this.filteredExerciseProgresses = this.query 
      ? this.filteredExerciseProgresses.filter(
          r => r.exercise.name.toLowerCase().indexOf(this.query.toLowerCase()) !== -1 ||
          r.taskInstances.some(ti => ti.task.name.toLocaleLowerCase().indexOf(this.query.toLowerCase()) !== -1)
      )
      : this.filteredExerciseProgresses;
  }
}
