import { Pipe, PipeTransform } from "@angular/core";
import { LruCache } from "app/shared/helper/lru-cache";
import { Exercise } from "../models/exercise";
import { Submission } from "../models/submission";
import { TaskInstance } from "../models/task-instance";

const isExercise = (b: Exercise | TaskInstance): b is Exercise => {
    return (b as Exercise).name !== undefined
}

@Pipe({
    name: 'points',
    pure: false
})
export class PointsPipe implements PipeTransform {
    cache: LruCache<string> = new LruCache<string>();

    transform(val: TaskInstance | Exercise, submission: Submission): any {
        let formatted = "";

        let cached = this.cache.get(`${submission?.totalPoints}`);
        if (cached) {
            return cached;
        }

        if (isExercise(val)) {
            if (submission == null) {
                formatted = `${val.reachablePoints} Punkte`;
            } else {
                var reachedPoints = submission.totalPoints ?? '-';
                formatted = `${reachedPoints} / ${val.reachablePoints} Punkte`;
            }
        } else {
            if (submission == null) {
                formatted = `${val.reachablePoints} Punkte`;
            } else {
                var reachedPoints = submission.reachedPointsFor(val) ?? '-';
                formatted = `${reachedPoints} / ${val.reachablePoints} Punkte`;
            }
        }
        this.cache.put(`${submission?.totalPoints}`, formatted);
        return formatted;
    }
}