import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { Term } from '../../models/term';
import { TermService } from '../../services/term.service';

@Component({
  selector: 'ngx-term-list',
  templateUrl: './term-list.component.html',
  styleUrls: ['./term-list.component.scss']
})
export class TermListComponent extends BaseComponent implements OnInit {
  terms: Term[] = [];
  loading: boolean = true;
  error: BackendError;

  constructor(
    private termService: TermService,
    private router: Router,
    private authService: AuthService,
    private notificationService: NotificationService,
    private route: ActivatedRoute) {
    super();
  }

  ngOnInit(): void {
    this.termService.getAll()
    .subscribe(terms => {
      this.terms = terms;
      this.loading = false;
    }, error => {
      this.loading = false;

      if (BackendError.isBackendError(error)) {
        this.error = error;
        this.notificationService.showError(
          error.format(
            "Fehler beim Laden der Semester",
          )
        );
      } else {
        throw error;
      }
    });

  }

}
