import { Component, Input, OnInit } from '@angular/core';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { Exercise } from 'app/features/exercises/models/exercise';
import { ExerciseService } from 'app/features/exercises/services/exercise.service';
import { TaskService } from '../../services/task.service';
import { BackendError } from 'app/shared/models/backend-error';
import { takeUntil } from 'rxjs/operators';
import { Task } from 'app/features/tasks/models/task';
import { AuthService } from 'app/core/services/auth.service';
import { TaskInstance } from 'app/features/exercises/models/task-instance';
import { NotificationService } from 'app/shared/services/notification.service';
import { FileRef } from '../../models/file-ref';
import { FileType } from '../../models/file-type';

@Component({
  selector: 'task-files',
  templateUrl: './task-files.component.html',
  styleUrls: ['./task-files.component.scss']
})
export class TaskFilesComponent extends BaseComponent implements OnInit {
  @Input() task: Task;
  @Input() exercise: Exercise;
  @Input() taskInstance: TaskInstance;

  loading: boolean = true;
  error: BackendError;
  refs: FileRef[] = [];
  FileType = FileType;

  constructor(
    private notificationService: NotificationService,
    private authService: AuthService,
    private taskService: TaskService,
    private exerciseService: ExerciseService,
  ) {
    super();
  }

  ngOnInit(): void {
    this.refresh();
  }

  get solutionPublic() {
    return this.taskInstance?.solutionPublic ?? false;
  }

  isFileVisible(file: FileRef): boolean {
    return file.public || (this.solutionPublic && file.type == FileType.Solution);
  }

  refresh(): void {
    this.loading = true;
    
    var loadFiles = this.exercise != null ? 
      this.exerciseService.getTaskInstanceFiles(this.exercise.id, this.task.id)
      : this.taskService.getFiles(this.task.id);

    loadFiles.pipe(
        takeUntil(this.unsubscribe),
    ).subscribe(files => {
      this.loading = false;
      this.refs = files;
    }, (error: any) => {
      if (BackendError.isBackendError(error)) {
        this.error = error;
        this.loading = false;
      } else {
        throw error;
      }
    });
  }

  downloadFile(file: FileRef) {
    var downloadFile = this.exercise != null ? 
      this.exerciseService.downloadFile(this.exercise.id, this.task.id, file)
      : this.taskService.downloadFile(this.task.id, file)

    downloadFile
    .subscribe(() => {}, error => {
      if (BackendError.isBackendError(error)) {
        this.notificationService.showError(
          error.format(
            "Fehler beim Herunterladen der Datei",
          )
        );
      } else {
        throw error;
      }
    })
  }
}
