import { Component, Input, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { takeUntil } from 'rxjs/operators';
import { TaskImportLog } from '../../models/task-import-log';
import { TaskPool } from '../../models/task-pool';
import { TaskPoolService } from '../../services/task-pool.service';

@Component({
  selector: 'ngx-task-import-logs',
  templateUrl: './task-import-logs.component.html',
  styleUrls: ['./task-import-logs.component.scss']
})
export class TaskImportLogsComponent extends BaseComponent implements OnInit {
  @Input() pool: TaskPool;
  logs: TaskImportLog[] = [];

  loading: boolean = true;
  error: BackendError;

  constructor(
    private taskPoolService: TaskPoolService,
    private router: Router,
    private route: ActivatedRoute) {
    super();
  }

  ngOnInit(): void {
    this.refresh();
  }

  public refresh() {
    this.taskPoolService.getLogs(this.pool.id)
      .pipe(
        takeUntil(this.unsubscribe)
      )
      .subscribe(logs => {
        this.logs = logs;
        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }

}
