import { HttpErrorResponse } from "@angular/common/http";
import { BackendError } from "app/shared/models/backend-error";
import { BaseComponent } from "./base.component";
import { FormMode } from "./form-mode";

export interface ValidationError {
    property: string;
    message: string;
}

export class FormComponent extends BaseComponent {
    mode: FormMode = null;
    errors: ValidationError[] = [];

    constructor() {
        super();
    }

    get isRead() {
        return this.mode == FormMode.Read;
    }

    get isNew() {
        return this.mode == FormMode.New;
    }

    get isEdit() {
        return this.mode == FormMode.Edit;
    }

    isInvalid(form, controlName) {
        return ((form.get(controlName).touched || form.get(controlName).dirty) && form.get(controlName).invalid) || this.errors.find(e => e.property == controlName);
    }

    getStatusForControl(form, controlName) {
        return this.isInvalid(form, controlName) ? 'danger' : 'basic';
    }

    public hasRequiredError(form, controlName) {
        return this.isInvalid(form, controlName) && form.get(controlName).errors?.['required'];
    }

    public hasFormError(form, err) {
        return form.errors?.[err];
    }

    public getValidationErrors(error: BackendError): ValidationError[] {
        let propertyMap = error.data;
        var validationErrors: ValidationError[] = [];

        if (error.isValidation) {
            // { description: 'blabla this is wrong' }
            for (const [key, value] of Object.entries(propertyMap)) {
                validationErrors.push({ property: key, message: <string>value })
            }
        }
        return validationErrors;
    }
}