import { Component, EventEmitter, Input, OnInit, Output, SimpleChanges, ViewChild } from '@angular/core';
import { User } from 'app/core/models/user';
import { UserService } from 'app/core/services/user.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { Subject } from 'rxjs';
import { debounceTime, flatMap, switchMap, takeUntil } from 'rxjs/operators';

@Component({
  selector: 'ngx-user-picker',
  templateUrl: './user-picker.component.html',
  styleUrls: ['./user-picker.component.scss']
})
export class UserPickerComponent extends BaseComponent implements OnInit {
  @Output() userSelected = new EventEmitter<User>();
  _selectedUser: User | null = null;
  @Input() readonly: boolean = false;

  @ViewChild('autoInput') inputField;

  input: string;

  users: User[] = [];
  query: string;
  searching: boolean = false;
  searchingError: BackendError;
  private queryChanged: Subject<string> = new Subject<string>();

  constructor(
    private userService: UserService, 
    private notificationService: NotificationService,
    ) {
    super();
  }

  @Input() get selectedUser(): User | null {
    return this._selectedUser;
  }

  set selectedUser(val: User) {
    this._selectedUser = val;
    this.input = this.selectedUser?.name ?? '';
  }


  ngOnInit(): void {
    this.queryChanged
      .pipe(
        takeUntil(this.unsubscribe),
        debounceTime(500),
        switchMap(query => {
          this.searching = true;
          return this.userService.findAll({ q: query, active: true });
        })
      )
      .subscribe(users => {
        this.users = users.content;
        this.searching = false;
      }, error => {
        this.searching = false;
        if (BackendError.isBackendError(error)) {
          this.searchingError = error;
          this.notificationService.showError(
            error.format(
              "Fehler bei der Benutzersuche",
            )
          );
        } else {
          throw error;
        }
      });
  }

  onChange() {
    if(this.selectedUser != null) {
      this.selectedUser = null;
      this.userSelected.emit(null);
    }
    this.queryChanged.next(this.inputField.nativeElement.value);
  }

  fn(option) {
    if (typeof option == "string") {
      return option
    } else {
      return option.name;
    }
  }

  onSelectionChange(selectedValue) {
    if(selectedValue == null || typeof selectedValue == 'string') return;
    this.selectedUser = selectedValue;
    this.userSelected.emit(selectedValue);
  }
}
