import { HttpErrorResponse } from "@angular/common/http";

export class BackendError {
    type: string;
    reason: string;
    data: any;
    
    constructor(type: string, reason: string, data: any = {}) {
        this.type = type;
        this.reason = reason;
        this.data = data;
    }

    static notFound(reason: string) {
        return new BackendError('notFound', reason);
    }

    isNotFound() {
        return this.type == 'notFound';
    }

    static unknown(reason: string) {
        return new BackendError('internal', reason);
    }

    isUnknown() {
        return this.type == 'internal';
    }

    static validation(reason: string, data: any) {
        return new BackendError('validation', reason, data);
    }

    isValidation() {
        return this.type == 'validation';
    }

    static dangerousAction(reason: string, data: any) {
        return new BackendError('dangerousAction', reason, data);
    }

    isDangerousAction() {
        return this.type == 'dangerousAction';
    }

    static accessDenied(reason: string) {
        return new BackendError('accessDenied', reason);
    }

    isAccessDenied() {
        return this.type == 'accessDenied';
    }

    public static parse(error: any): BackendError {
        let reason = this.getReason(error);

        if(this.isNotFound(error)) {
            return BackendError.notFound(reason);
        }

        if(this.isValidationError(error)) {
            let data = error.error.data;
            return BackendError.validation(reason, data);
        }

        if(this.isAccessDenied(error)) {
            return BackendError.accessDenied(reason);
        }

        if(this.isDangerousActionError(error)) {
            let data = error.error.data;
            return BackendError.dangerousAction(reason, data);
        }

        return BackendError.unknown(reason);
    }

    public static isValidationError(error: any): boolean {
        return error instanceof HttpErrorResponse && error.error.errorCode === 0;
    }

    public static isDangerousActionError(error: any): boolean {
        return error instanceof HttpErrorResponse && error.error.errorCode === 1;
    }

    public static isAccessDenied(error: any): boolean {
        if(error instanceof HttpErrorResponse) {
            return error.status == 403 || error.status == 401;
        }

        return false;
    }

    public static isNotFound(error: any): boolean {
        if(error instanceof HttpErrorResponse) {
            return error.status == 404;
        }

        return false;
    }

    public static getReason(error: any): string {
        if(error instanceof HttpErrorResponse) {
            return error.error.reasonPhrase;
        }

        return null;
    }

    public format(prefix: string): string {
        let message = this.reason ?? "Unbekannter Fehler";
        return `${prefix ?? "Fehler"}: ${message}`;
    }

    public static isBackendError(error: any): error is BackendError {
        return error.type !== undefined;
    }
}