import { HttpClient, HttpResponse } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { withCache } from "@ngneat/cashew";
import { Observable } from "rxjs";
import { catchError, map } from "rxjs/operators";
import { BackendError } from "../models/backend-error";

@Injectable({
    providedIn: 'root'
})
export class DownloadService {
    constructor(private http: HttpClient) {
    }

    // https://stackoverflow.com/questions/52154874/angular-6-downloading-file-from-rest-api
    downloadFileAsUrl(url: string, useCache: boolean = false): Observable<[url: string, fileName: string]> {
        return this.http.get(url, {
            observe: 'response',
            responseType: 'blob' as 'json',
            context: useCache ? withCache() : null
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map((response: HttpResponse<Blob>) => {
                let blob = new Blob([response.body], { type: response.body.type });
                var downloadURL = window.URL.createObjectURL(blob);
                let contentDisposition = response.headers.get('content-disposition');
                let fileName = contentDisposition.split(';')[1].split('filename')[1].split('=')[1].trim().replace(new RegExp("\"", 'g'), "");
                let decodedFileName = decodeURI(fileName);
                return [downloadURL, decodedFileName];
            })
        );
    }

    downloadFile(url: string, useCache: boolean = false): Observable<void> {
        return this.downloadFileAsUrl(url, useCache).pipe(
            map(([url, fileName]) => {
                var link = document.createElement('a');
                link.href = url;
                link.download = fileName;
                link.click();
            })
        )
    }
}