package de.hsrm.sls.subato.aor.core;

import de.hsrm.sls.subato.aor.core.imp.DegreeProgramImportService;
import de.hsrm.sls.subato.aor.core.imp.ProgramModuleImportService;
import de.hsrm.sls.subato.modules.program.DegreeProgramRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.stream.Collectors;

import static de.hsrm.sls.subato.aor.core.imp.ModuleFrequencyConverter.convertFrequency;

@Service
@Transactional
public class CoreDataImportManager {
    @Autowired
    DegreeProgramRepository programRepository;

    @Autowired
    DegreeProgramImportService degreeProgramImportService;

    @Autowired
    ProgramModuleImportService programModuleImportService;

    @Autowired
    JsonCoreDataReader jsonCoreDataReader;

    public void importFromJson(String json) {
        var coreData = jsonCoreDataReader.read(json);

        degreeProgramImportService.importPrograms(coreData.getCoursePrograms());

        var existingPrograms = programRepository.findAll();
        var filteredModuleNodes = coreData.getModuleNodes().stream()
                .filter(n -> existingPrograms.stream().anyMatch(p -> n.getProgramId().equals(p.getAorId())))
                //.filter(n -> n.getListId() == null)
                .filter(n -> convertFrequency(n.getFrequency()) != null)
                .filter(n -> n.getModuleParts().stream().anyMatch(mp -> mp.getCourseType() != null && (mp.getCourseType().getMatchcode().equals("P") || mp.getCourseType().getMatchcode().equals("Ü"))))
                .collect(Collectors.toList());

        programModuleImportService.importModules(filteredModuleNodes);
    }
}
