package de.hsrm.sls.subato.aor.term;

import de.hsrm.sls.subato.aor.term.imp.unit.CourseUnitImportService;
import de.hsrm.sls.subato.aor.term.persistence.TermDataImportService;
import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.courses.course.CourseSupplement;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class AoRCourseSupplement implements CourseSupplement {
    @Autowired
    CourseUnitImportService courseUnitImportService;

    @Autowired
    TermDataImportService termDataImportService;

    @Autowired
    JsonTermDataReader jsonTermDataReader;

    public boolean isLinkedToAoR(Course course) {
        return course.getModules().size() != 0 && course.getTerm() != null && course.getTerm().getTermDataImport() != null && course.getTerm().getTermDataImport().size() != 0;
    }

    @Override
    public Course supplement(Course course) {
        if(course.getTerm() == null || !isLinkedToAoR(course)) {
            return course;
        }

        var termDataImport = termDataImportService.getTermData(course.getTerm());
        var termData = jsonTermDataReader.read(termDataImport.getFile());
        courseUnitImportService.importUnits(course, termData);

        return course;
    }

    @Override
    public String getIdentifier() {
        return "aor";
    }
}
