package de.hsrm.sls.subato.auth;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Querschnitt")
public class AuthController {
    @Autowired
    private PrincipalService principalService;

    @GetMapping("/auth/principal")
    @PreAuthorize("isAuthenticated()")
    @Operation(summary = "Liefert Informationen zum aktuell eingeloggten Benutzer")
    SubatoPrincipal getCurrentUser() {
        return principalService.getPrincipal();
    }

    @PostMapping("/auth/logout")
    @PreAuthorize("isAuthenticated()")
    @Operation(summary = "Löscht den eingeloggten Benutzer aus dem Cache")
    void logout() {
        var principal = principalService.getPrincipal();
        principalService.evictPrincipal(principal);
    }
}
