package de.hsrm.sls.subato.auth.masking;

import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.user.UserDetailDto;
import de.hsrm.sls.subato.user.UserSummaryDto;
import org.springframework.stereotype.Component;

@Component
public class UserMasker implements Masker {
    @Override
    public void mask(Object obj, SubatoPrincipal principal) {
        if(obj instanceof UserDetailDto) {
            mask((UserDetailDto) obj, principal);
        }

        if(obj instanceof UserSummaryDto) {
            mask((UserSummaryDto) obj, principal);
        }
    }

    private void mask(UserDetailDto obj, SubatoPrincipal principal) {
        if(principal == null) {
            obj.setMail(null);
        }

        if(principal == null || !(obj.getId() == principal.getId() || principal.hasAnyRole(Role.Staff, Role.SuperAdmin) || principal.hasAnyGroup())) {
            obj.setLoginName(null);
        }
    }

    private void mask(UserSummaryDto obj, SubatoPrincipal principal) {
        if(principal == null) {
            obj.setMail(null);
        }

        if(principal == null || !(obj.getId() == principal.getId() || principal.hasAnyRole(Role.Staff, Role.SuperAdmin) || principal.hasAnyGroup())) {
            obj.setLoginName(null);
        }
    }

    @Override
    public boolean supports(Class clazz) {
        return clazz.equals(UserDetailDto.class) || clazz.equals(UserSummaryDto.class);
    }
}
