package de.hsrm.sls.subato.courses.course;

import de.hsrm.sls.subato.courses.membership.StaffGroup;
import de.hsrm.sls.subato.courses.membership.CourseStaffMembership;
import de.hsrm.sls.subato.courses.unit.CourseUnitType;
import de.hsrm.sls.subato.exercises.exercise.Exercise;
import de.hsrm.sls.subato.courses.unit.CourseUnit;
import de.hsrm.sls.subato.courses.file.CourseFile;
import de.hsrm.sls.subato.modules.progmodule.ProgramModule;
import de.hsrm.sls.subato.modules.term.Term;
import de.hsrm.sls.subato.user.User;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Entity
@Table(name = "course")
public class Course {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    private String name;

    private String shortName;

    @Column(length = 10000000)
    private String description;

    private String motd;

    @NotNull
    private CourseVisibility visibility;

    @NotNull
    private SubmitPolicy submitPolicy;

    @ManyToMany
    private List<ProgramModule> modules;

    @OneToMany(mappedBy = "course", cascade = CascadeType.REMOVE)
    private List<Exercise> exercises = new ArrayList<>();

    @ManyToOne
    private Term term;

    @OneToMany(mappedBy = "course", cascade = CascadeType.ALL, orphanRemoval=true)
    private List<CourseStaffMembership> memberships = new ArrayList<>();

    @OneToMany(mappedBy = "course", cascade = CascadeType.ALL, orphanRemoval=true)
    private List<CourseUnit> units = new ArrayList<>();

    @OneToMany(mappedBy = "course")
    private List<CourseFile> files;

    @Column(unique = true)
    private Long sourceId;

    private boolean showEmails;

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getShortName() {
        return shortName;
    }

    public void setShortName(String shortName) {
        this.shortName = shortName;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public List<ProgramModule> getModules() {
        return modules;
    }

    public void setModules(List<ProgramModule> modules) {
        this.modules = modules;
    }

    public Term getTerm() {
        return term;
    }

    public void setTerm(Term term) {
        this.term = term;
    }

    public List<CourseStaffMembership> getMemberships() {
        return memberships;
    }

    public void setMemberships(List<CourseStaffMembership> groupMemberships) {
        this.memberships = groupMemberships;
    }

    public List<CourseUnit> getUnits() {
        return units;
    }

    public void setUnits(List<CourseUnit> units) {
        this.units = units;
    }

    public List<CourseFile> getFiles() {
        return files;
    }

    public void setFiles(List<CourseFile> files) {
        this.files = files;
    }

    public List<User> getAdmins() {
        return this.memberships.stream()
                .filter(m -> m.getGroup().equals(StaffGroup.ADMIN))
                .map(m -> m.getUser())
                .collect(Collectors.toList());
    }

    public List<User> getTutors() {
        return this.memberships.stream()
                .filter(m -> m.getGroup().equals(StaffGroup.TUTOR))
                .map(m -> m.getUser())
                .collect(Collectors.toList());
    }

    public List<User> getLecturers() {
        return this.memberships.stream()
                .filter(m -> m.getGroup().equals(StaffGroup.LECTURER))
                .map(m -> m.getUser())
                .collect(Collectors.toList());
    }

    public Long getSourceId() {
        return sourceId;
    }

    public void setSourceId(Long sourceId) {
        this.sourceId = sourceId;
    }

    public List<Exercise> getExercises() {
        return exercises;
    }

    public void setExercises(List<Exercise> exercises) {
        this.exercises = exercises;
    }

    public String getMotd() {
        return motd;
    }

    public void setMotd(String motd) {
        this.motd = motd;
    }

    public CourseVisibility getVisibility() {
        return visibility;
    }

    public void setVisibility(CourseVisibility visibility) {
        this.visibility = visibility;
    }

    public SubmitPolicy getSubmitPolicy() {
        return submitPolicy;
    }

    public void setSubmitPolicy(SubmitPolicy submitPolicy) {
        this.submitPolicy = submitPolicy;
    }

    public boolean isShowEmails() {
        return showEmails;
    }

    public void setShowEmails(boolean showEmails) {
        this.showEmails = showEmails;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Course course = (Course) o;
        return id == course.id;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }



    public CourseUnit getSandbox() {
        return units.stream().filter(u -> u.getType() == CourseUnitType.Misc && u.getName().equals("Sandkasten")).findFirst().orElse(null);
    }

    public CourseUnit getDefault() {
        return units.stream().filter(u -> u.getType() == CourseUnitType.Misc && u.getName().equals("Standard")).findFirst().orElse(null);
    }
}
