package de.hsrm.sls.subato.dashboard;

import de.hsrm.sls.subato.attendances.attendance.AttendanceService;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.courses.course.CourseService;
import de.hsrm.sls.subato.courses.unit.CourseUnitService;
import de.hsrm.sls.subato.courses.unit.CourseUnitType;
import de.hsrm.sls.subato.exercises.submission.progress.SubmissionProgressService;
import de.hsrm.sls.subato.user.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.stream.Collectors;

@Service
public class DashboardService {
    @Autowired
    private CourseService courseService;

    @Autowired
    private CourseUnitService courseUnitService;

    @Autowired
    private AttendanceService attendanceService;

    @Autowired
    private SubmissionProgressService submissionProgressService;

    @Autowired
    private UserService userService;

    @Autowired
    private PolicyEvaluator policy;

    public DashboardState generateDashboard(Long termId, SubatoPrincipal principal) {
        var user = userService.findByPrincipal(principal);

        var courseInfos = new ArrayList<CourseInfo>();
        var memberCourses = courseService.findCoursesWhereIsMember(user, termId);
        var participatingCourses = courseService.findParticipatingCourses(user, termId);

        for(var course : participatingCourses) {
            if(memberCourses.contains(course)) {
                continue;
            }

            var canViewCourse = policy.check(new CheckPolicyRequest(principal, course, CoursesAction.COURSE_VIEW, new HashMap<>()));
            if(!canViewCourse) {
                continue;
            }

            var participatingUnits = courseUnitService.getParticipatingUnits(course, user);
            participatingUnits = participatingUnits.stream().filter(u -> u.getType() != CourseUnitType.Misc).collect(Collectors.toList());
            var courseInfo = new CourseInfo();
            courseInfo.setParticipant(true);
            courseInfo.setCourse(course);
            courseInfo.setUnits(participatingUnits);

            var submissionsResult = submissionProgressService.getProgressForUserInCourse(course, user);
            courseInfo.setSubmissionsProgress(submissionsResult);

            var participantAttendances = attendanceService.getAttendanceForCourseAndUser(course, user);
            courseInfo.setParticipantAttendances(participantAttendances);

            courseInfos.add(courseInfo);
        }

        for(var course : memberCourses) {
            var lecturingUnits = courseUnitService.getLecturingUnits(user, course);

            var courseInfo = new CourseInfo();
            courseInfo.setParticipant(false);
            courseInfo.setCourse(course);
            courseInfo.setUnits(lecturingUnits);

            courseInfos.add(courseInfo);
        }

        return new DashboardState(courseInfos);
    }
}
