package de.hsrm.sls.subato.exercises.submission;

import de.hsrm.sls.subato.courses.course.Course_;
import de.hsrm.sls.subato.courses.unit.CourseUnit_;
import de.hsrm.sls.subato.exercises.exercise.Exercise_;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.User_;
import org.springframework.data.jpa.domain.Specification;

import javax.persistence.criteria.Expression;
import javax.persistence.criteria.JoinType;
import java.util.Set;

public class SubmissionSpecifications {
    public static Specification<Submission> submittedByEquals(long submittedById) {
        return (ex, cq, cb) -> cb.equal(ex.get(Submission_.submittedBy).get(User_.id), submittedById);
    }

    public static Specification<Submission> exerciseEquals(long exerciseId) {
        return (ex, cq, cb) -> cb.equal(ex.get(Submission_.exercise).get(Exercise_.id), exerciseId);
    }

    public static Specification<Submission> unitEquals(long unitId) {
        return (root, query, cb) -> {
            var submissionUser = root.join(Submission_.submittedBy, JoinType.INNER);
            var userCourseUnit = submissionUser.join(User_.participatingUnits, JoinType.INNER);
            var participants = userCourseUnit.get(CourseUnit_.participants);
            userCourseUnit = userCourseUnit.on(cb.isMember(root.get(Submission_.submittedBy), participants));

            query.distinct(true);
            return cb.equal(userCourseUnit.get(CourseUnit_.id), unitId);
        };
    }

    public static Specification<Submission> courseEquals(long courseId) {
        return (root, query, cb) -> {
            var submissionExercise = root.join(Submission_.exercise, JoinType.INNER);
            return cb.equal(submissionExercise.get(Exercise_.course).get(Course_.id), courseId);
        };
    }
}
