package de.hsrm.sls.subato.exercises.taskinstance;

import de.hsrm.sls.subato.tasks.stef.file.FileRef;
import de.hsrm.sls.subato.tasks.task.TaskService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;

import static de.hsrm.sls.subato.exercises.taskinstance.TaskInstanceSpecifications.*;

@Service
public class TaskInstanceService {
    @Autowired
    private TaskInstanceRepository taskInstanceRepository;

    @Autowired
    private TaskService taskService;

    /**
     * @throws TaskInstanceNotFoundException
     */
    public TaskInstance findByExerciseAndTask(long exerciseId, long taskId) {
        var taskInstances = findAll(
                new TaskInstanceFilter()
                        .exerciseId(exerciseId)
                        .taskId(taskId)
        );

        return taskInstances.stream().findFirst().orElseThrow(() -> new TaskInstanceNotFoundException());
    }

    /**
     * gets only these task instances which are associated with a task
     * that was not archived. archived task instances should not be visible.
     */
    public List<TaskInstance> findAll(TaskInstanceFilter filter) {
        Specification<TaskInstance> spec = filter.toSpecification();
        spec = spec.and(notArchived());

        var taskInstances = taskInstanceRepository.findAll(spec);
        taskInstances.sort(Comparator.comparing(TaskInstance::getOrderNumber));
        return taskInstances;
    }

    public void publishSolution(TaskInstance taskInstance, boolean publish) {
        taskInstance.setSolutionPublic(publish);
        taskInstanceRepository.save(taskInstance);
    }

    public List<FileRef> getFiles(TaskInstance taskInstance) {
        var files = taskService.getFiles(taskInstance.getTask());
        return files;
    }
}
