package de.hsrm.sls.subato.solutions.cleanup;

import de.hsrm.sls.subato.solutions.solution.SolutionRepository;
import de.hsrm.sls.subato.solutions.solution.SolutionService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * responsible for cleaning up anonymous solutions
 */
@Service
public class SolutionCleanupService {
    private static Logger logger = LoggerFactory.getLogger(SolutionCleanupService.class);

    @Autowired
    private SolutionRepository solutionRepository;

    @Autowired
    private SolutionService solutionService;

    public void cleanupAnonymousSolution() {
        var now = LocalDateTime.now();
        var anonymousSolutions = solutionRepository.findAllBySubmittedByIsNull();

        var deletedSolutions = 0;
        for(var solution : anonymousSolutions) {
            if(now.isAfter(solution.getSubmittedAt().plusMinutes(5))) {
                solutionService.delete(solution);
                deletedSolutions += 1;
            }
        }

        logger.info("%s/%s anonymous solutions were deleted".formatted(deletedSolutions, anonymousSolutions.size()));
    }
}
