package de.hsrm.sls.subato.solutions.evaluation.request;

import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.shared.fs.FileTree;
import de.hsrm.sls.subato.tasks.stef.file.FileRef;
import org.junit.Assert;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;


@SpringBootTest
@Tag(Tags.UNIT)
public class FirstFileMatcherTest {
    @Test
    public void shouldMatchIfOneItem() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        solutionTree.getChildren().add(new FileTree("First.java", "First.java", "text/x-java-source", true));

        var matcher = new FirstFileMatcher();
        var tree = matcher.match(solutionTree, templateFile);

        Assert.assertNotNull(tree);
    }

    @Test
    public void shouldMatchIfOneItemAndDifferentName() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        solutionTree.getChildren().add(new FileTree("First (39).java", "First (39).java", "text/x-java-source", true));

        var matcher = new FirstFileMatcher();
        var tree = matcher.match(solutionTree, templateFile);

        Assert.assertNotNull(tree);
    }

    @Test
    public void shouldNotMatchIfMultipleItems() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        solutionTree.getChildren().add(new FileTree("First.java", "First.java", "text/x-java-source", true));
        solutionTree.getChildren().add(new FileTree("Second.java", "Second.java", "text/x-java-source", true));

        var matcher = new FirstFileMatcher();
        var tree = matcher.match(solutionTree, templateFile);

        Assert.assertNull(tree);
    }

    @Test
    public void shouldNotMatchIfDirectory() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        var dir = new FileTree("ABC", "ABC", "", false);
        dir.getChildren().add(new FileTree("First.java", "ABC/First.java", "text/x-java-source", true));
        solutionTree.getChildren().add(dir);

        var matcher = new FirstFileMatcher();
        var tree = matcher.match(solutionTree, templateFile);

        Assert.assertNull(tree);
    }
}
